from unittest import TestCase

from tests.mock_client import example_vessel_movements
from vortexasdk.api import (
    CorporateEntity,
    GeographyEntity,
    VesselEntity,
    VesselMovement,
)
from vortexasdk.api.entity_flattening import (
    convert_vessel_movement_to_flat_dict,
)
from vortexasdk.api.serdes import serialize_to_dict
from vortexasdk.api.vessel_movement import VesselEvent


class TestVesselMovement(TestCase):
    vm = VesselMovement(
        vessel_movement_id="7e0fb931ec1117d102f957048a6cce473a7eaf3f2b95f6bccdcc7b50dc3d8ab7",
        voyage_id="7a29c4880276cbbdfade2ec6a0ee2dbbebf9e41bb47b18657196fff7cd3dc32e",
        vessel=VesselEntity(
            id="6d8a8f0863ca087204dd68e5fc3b6469a879829e6262856e34856aea3ca20509",
            name="17 FEBRUARY",
            mmsi=248896000,
            imo=9380893,
            dwt=160391,
            vessel_class="suezmax",
            corporate_entities=[
                CorporateEntity(
                    id="19b1bf392f474ca942b7b25ddcba363638516b2e40268ea7a70888021575a4dd",
                    label="VITOL",
                    layer="charterer",
                    probability=1.0,
                    source="external",
                ),
                CorporateEntity(
                    id="01ec474e7324c7633ffaf2a5c92bb7348cc02adc4d10b85daa3dd2cc5458df59",
                    label="CORE PETROLEUM",
                    layer="commercial_owner",
                    probability=1.0,
                    source="external",
                ),
            ],
            tags=[],
            status="vessel_status_ballast",
            start_timestamp=None,
            cubic_capacity=172092,
            voyage_id=None,
            fixture_fulfilled=None,
            end_timestamp=None,
            fixture_id=None,
        ),
        origin=VesselEvent(
            event_type="discharge",
            location=[
                GeographyEntity(
                    id="934c47f36c16a58d68ef5e007e62a23f5f036ee3f3d1f5f85a48c572b90ad8b2",
                    label="China",
                    layer="country",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="c901cc99a794afe1a7a9146c4573420e87c629717651c3a0d2ee3628cdaf0bc5",
                    label="Bayuquan [CN]",
                    layer="port",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="e6955a2c59dc90833986fe0894cf6718dddaa7816bb51bc955cdd3eb4470e554",
                    label="Asia",
                    layer="region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="07ac020aa082d5d116f829ef28a40e9876c0644aeb4f0480202c1a86bb8adf7f",
                    label="Far East",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="2b394b41a7ff9e48b40b4ba8e6a6b045340261dfc02133d04af13fe754a71a48",
                    label="East Asia",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="a63890260e29d859390fd1a23c690181afd4bd152943a04c00cd6a5ecf3f7d1e",
                    label="North China",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="b5fafce6e20de2dc307fb7e0b89978ee91a49a7b6ec6f5461daf2633f3c56674",
                    label="China (excl. HK & Macau)",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="956b03529510459d298c4e537d23c6fbb22ffd8414af933b3475310c661f337a",
                    label="Non-OPEC",
                    layer="trading_block",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="2b394b41a7ff9e48b40b4ba8e6a6b045340261dfc02133d04af13fe754a71a48",
                    label="East Asia",
                    layer="trading_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="87f8d059ed2f57c1f54dd883cb0fc90f83633d736f763982f2503f6f803eb333",
                    label="Liaoning Province",
                    layer="trading_subregion",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="977573e3782428763edf580be53c73b7d04172037e6d6ba8282ea1de66dd36cb",
                    label="Xianrendao Crude Oil Terminal",
                    layer="terminal",
                    probability=1.0,
                    source="model",
                ),
            ],
            pos=[121.95271103090423, 40.20963715299812],
            event_id="5769995a932e152f9ded42706cebe90992ba6cc06208e32833d29ea91e37a68f",
            start_timestamp="2017-11-19T06:59:05+0000",
            end_timestamp="2017-11-21T00:00:08+0000",
        ),
        destination=VesselEvent(
            event_type="discharge",
            location=[
                GeographyEntity(
                    id="934c47f36c16a58d68ef5e007e62a23f5f036ee3f3d1f5f85a48c572b90ad8b2",
                    label="China",
                    layer="country",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="056054a06d321e21ac868558a099751b324b675e05c11df7c041c8ad0edf5445",
                    label="Dalian [CN]",
                    layer="port",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="e6955a2c59dc90833986fe0894cf6718dddaa7816bb51bc955cdd3eb4470e554",
                    label="Asia",
                    layer="region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="07ac020aa082d5d116f829ef28a40e9876c0644aeb4f0480202c1a86bb8adf7f",
                    label="Far East",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="2b394b41a7ff9e48b40b4ba8e6a6b045340261dfc02133d04af13fe754a71a48",
                    label="East Asia",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="a63890260e29d859390fd1a23c690181afd4bd152943a04c00cd6a5ecf3f7d1e",
                    label="North China",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="b5fafce6e20de2dc307fb7e0b89978ee91a49a7b6ec6f5461daf2633f3c56674",
                    label="China (excl. HK & Macau)",
                    layer="shipping_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="956b03529510459d298c4e537d23c6fbb22ffd8414af933b3475310c661f337a",
                    label="Non-OPEC",
                    layer="trading_block",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="2b394b41a7ff9e48b40b4ba8e6a6b045340261dfc02133d04af13fe754a71a48",
                    label="East Asia",
                    layer="trading_region",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="87f8d059ed2f57c1f54dd883cb0fc90f83633d736f763982f2503f6f803eb333",
                    label="Liaoning Province",
                    layer="trading_subregion",
                    probability=1.0,
                    source="model",
                ),
                GeographyEntity(
                    id="e4694abfad48f9135cdd6634ebb0624c3a91097fc8e83b084747d5bdbd8c2c8e",
                    label="Dalian Xingang Oil Terminal",
                    layer="terminal",
                    probability=1.0,
                    source="model",
                ),
            ],
            pos=[121.9107237155891, 38.965178213026206],
            event_id="487d3eba7cc29724ce6b8aca3822dfbe85eaa0a61500c3cd5fa5f705bf95c832",
            start_timestamp="2017-11-27T06:39:50+0000",
            end_timestamp="2017-11-29T02:49:32+0000",
        ),
        start_timestamp="2017-11-19T06:59:05+0000",
        end_timestamp="2017-11-29T02:49:32+0000",
    )

    def test_deserialize(self):
        vm1 = example_vessel_movements[0]

        deserialized = VesselMovement.from_dict(vm1)

        assert self.vm == deserialized

    def test_flatten(self):
        d = serialize_to_dict(self.vm)
        flat = convert_vessel_movement_to_flat_dict(d)

        expected = {
            "destination.end_timestamp": "2017-11-29T02:49:32+0000",
            "destination.event_id": "487d3eba7cc29724ce6b8aca3822dfbe85eaa0a61500c3cd5fa5f705bf95c832",
            "destination.event_type": "discharge",
            "destination.location.country.id": "934c47f36c16a58d68ef5e007e62a23f5f036ee3f3d1f5f85a48c572b90ad8b2",
            "destination.location.country.label": "China",
            "destination.location.country.layer": "country",
            "destination.location.country.probability": 1.0,
            "destination.location.country.source": "model",
            "destination.location.port.id": "056054a06d321e21ac868558a099751b324b675e05c11df7c041c8ad0edf5445",
            "destination.location.port.label": "Dalian [CN]",
            "destination.location.port.layer": "port",
            "destination.location.port.probability": 1.0,
            "destination.location.port.source": "model",
            "destination.location.region.id": "e6955a2c59dc90833986fe0894cf6718dddaa7816bb51bc955cdd3eb4470e554",
            "destination.location.region.label": "Asia",
            "destination.location.region.layer": "region",
            "destination.location.region.probability": 1.0,
            "destination.location.region.source": "model",
            "destination.location.shipping_region.id": "b5fafce6e20de2dc307fb7e0b89978ee91a49a7b6ec6f5461daf2633f3c56674",
            "destination.location.shipping_region.label": "China (excl. HK & Macau)",
            "destination.location.shipping_region.layer": "shipping_region",
            "destination.location.shipping_region.probability": 1.0,
            "destination.location.shipping_region.source": "model",
            "destination.location.trading_block.id": "956b03529510459d298c4e537d23c6fbb22ffd8414af933b3475310c661f337a",
            "destination.location.trading_block.label": "Non-OPEC",
            "destination.location.trading_block.layer": "trading_block",
            "destination.location.trading_block.probability": 1.0,
            "destination.location.trading_block.source": "model",
            "destination.location.trading_region.id": "2b394b41a7ff9e48b40b4ba8e6a6b045340261dfc02133d04af13fe754a71a48",
            "destination.location.trading_region.label": "East Asia",
            "destination.location.trading_region.layer": "trading_region",
            "destination.location.trading_region.probability": 1.0,
            "destination.location.trading_region.source": "model",
            "destination.location.trading_subregion.id": "87f8d059ed2f57c1f54dd883cb0fc90f83633d736f763982f2503f6f803eb333",
            "destination.location.trading_subregion.label": "Liaoning Province",
            "destination.location.trading_subregion.layer": "trading_subregion",
            "destination.location.trading_subregion.probability": 1.0,
            "destination.location.trading_subregion.source": "model",
            "destination.location.terminal.id": "e4694abfad48f9135cdd6634ebb0624c3a91097fc8e83b084747d5bdbd8c2c8e",
            "destination.location.terminal.label": "Dalian Xingang Oil Terminal",
            "destination.location.terminal.layer": "terminal",
            "destination.location.terminal.probability": 1.0,
            "destination.location.terminal.source": "model",
            "destination.pos.0": 121.9107237155891,
            "destination.pos.1": 38.965178213026206,
            "destination.start_timestamp": "2017-11-27T06:39:50+0000",
            "end_timestamp": "2017-11-29T02:49:32+0000",
            "origin.end_timestamp": "2017-11-21T00:00:08+0000",
            "origin.event_id": "5769995a932e152f9ded42706cebe90992ba6cc06208e32833d29ea91e37a68f",
            "origin.event_type": "discharge",
            "origin.location.country.id": "934c47f36c16a58d68ef5e007e62a23f5f036ee3f3d1f5f85a48c572b90ad8b2",
            "origin.location.country.label": "China",
            "origin.location.country.layer": "country",
            "origin.location.country.probability": 1.0,
            "origin.location.country.source": "model",
            "origin.location.port.id": "c901cc99a794afe1a7a9146c4573420e87c629717651c3a0d2ee3628cdaf0bc5",
            "origin.location.port.label": "Bayuquan [CN]",
            "origin.location.port.layer": "port",
            "origin.location.port.probability": 1.0,
            "origin.location.port.source": "model",
            "origin.location.region.id": "e6955a2c59dc90833986fe0894cf6718dddaa7816bb51bc955cdd3eb4470e554",
            "origin.location.region.label": "Asia",
            "origin.location.region.layer": "region",
            "origin.location.region.probability": 1.0,
            "origin.location.region.source": "model",
            "origin.location.shipping_region.id": "b5fafce6e20de2dc307fb7e0b89978ee91a49a7b6ec6f5461daf2633f3c56674",
            "origin.location.shipping_region.label": "China (excl. HK & Macau)",
            "origin.location.shipping_region.layer": "shipping_region",
            "origin.location.shipping_region.probability": 1.0,
            "origin.location.shipping_region.source": "model",
            "origin.location.trading_block.id": "956b03529510459d298c4e537d23c6fbb22ffd8414af933b3475310c661f337a",
            "origin.location.trading_block.label": "Non-OPEC",
            "origin.location.trading_block.layer": "trading_block",
            "origin.location.trading_block.probability": 1.0,
            "origin.location.trading_block.source": "model",
            "origin.location.trading_region.id": "2b394b41a7ff9e48b40b4ba8e6a6b045340261dfc02133d04af13fe754a71a48",
            "origin.location.trading_region.label": "East Asia",
            "origin.location.trading_region.layer": "trading_region",
            "origin.location.trading_region.probability": 1.0,
            "origin.location.trading_region.source": "model",
            "origin.location.trading_subregion.id": "87f8d059ed2f57c1f54dd883cb0fc90f83633d736f763982f2503f6f803eb333",
            "origin.location.trading_subregion.label": "Liaoning Province",
            "origin.location.trading_subregion.layer": "trading_subregion",
            "origin.location.trading_subregion.probability": 1.0,
            "origin.location.trading_subregion.source": "model",
            "origin.location.terminal.id": "977573e3782428763edf580be53c73b7d04172037e6d6ba8282ea1de66dd36cb",
            "origin.location.terminal.label": "Xianrendao Crude Oil Terminal",
            "origin.location.terminal.layer": "terminal",
            "origin.location.terminal.probability": 1.0,
            "origin.location.terminal.source": "model",
            "origin.pos.0": 121.95271103090423,
            "origin.pos.1": 40.20963715299812,
            "origin.start_timestamp": "2017-11-19T06:59:05+0000",
            "start_timestamp": "2017-11-19T06:59:05+0000",
            "vessel_movement_id": "7e0fb931ec1117d102f957048a6cce473a7eaf3f2b95f6bccdcc7b50dc3d8ab7",
            "voyage_id": "7a29c4880276cbbdfade2ec6a0ee2dbbebf9e41bb47b18657196fff7cd3dc32e",
            "vessel.corporate_entities.charterer.id": "19b1bf392f474ca942b7b25ddcba363638516b2e40268ea7a70888021575a4dd",
            "vessel.corporate_entities.charterer.label": "VITOL",
            "vessel.corporate_entities.charterer.layer": "charterer",
            "vessel.corporate_entities.charterer.probability": 1.0,
            "vessel.corporate_entities.charterer.source": "external",
            "vessel.corporate_entities.commercial_owner.id": "01ec474e7324c7633ffaf2a5c92bb7348cc02adc4d10b85daa3dd2cc5458df59",
            "vessel.corporate_entities.commercial_owner.label": "CORE PETROLEUM",
            "vessel.corporate_entities.commercial_owner.layer": "commercial_owner",
            "vessel.corporate_entities.commercial_owner.probability": 1.0,
            "vessel.corporate_entities.commercial_owner.source": "external",
            "vessel.cubic_capacity": 172092,
            "vessel.dwt": 160391,
            "vessel.end_timestamp": None,
            "vessel.fixture_fulfilled": None,
            "vessel.fixture_id": None,
            "vessel.id": "6d8a8f0863ca087204dd68e5fc3b6469a879829e6262856e34856aea3ca20509",
            "vessel.imo": 9380893,
            "vessel.mmsi": 248896000,
            "vessel.name": "17 FEBRUARY",
            "vessel.start_timestamp": None,
            "vessel.status": "vessel_status_ballast",
            "vessel.vessel_class": "suezmax",
            "vessel.voyage_id": None,
            "vessel.year": None,
            "vessel.flag": None,
            "vessel.scrubber": None,
            "vessel.ice_class": None,
            "vessel.propulsion": None,
        }

        assert expected == flat



from unittest import TestCase

from vortexasdk.api.id import is_valid_id, split_ids_other


class TestID(TestCase):
    def test_is_id_recognises_id(self):
        assert is_valid_id(
            "2aaad41b89dfad19e5668918018ae02695d7710bcbe5f2dc689234e8da492de3"
        )

    def test_is_id_recognised_wrong_length_id(self):
        double_length_id = (
            "2aaad41b89dfad19e5668918018ae02695d7710bcbe5f2dc689234e8da492de3"
            + "2aaad41b89dfad19e5668918018ae02695d7710bcbe5f2dc689234e8da492de3"
        )
        assert not is_valid_id(double_length_id)

    def test_is_id_recognises_non_id(self):
        assert not is_valid_id("United Kingdom")

    def test_separate_ids_from_names(self):
        mix = [
            "2aaad41b89dfad19e5668918018ae02695d7710bcbe5f2dc689234e8da492de3",
            "China",
            "Southamton",
            "918018ae02695d7710bcbe5f2dc689234e8da492de32aaad41b89dfad19e5668",
        ]

        ids, names = split_ids_other(mix)

        assert ids == [
            "2aaad41b89dfad19e5668918018ae02695d7710bcbe5f2dc689234e8da492de3",
            "918018ae02695d7710bcbe5f2dc689234e8da492de32aaad41b89dfad19e5668",
        ]

        assert names == ["China", "Southamton"]



import time

from vortexasdk.logger import get_logger

logger = get_logger(__name__)


class Timer:
    """
    Time functions.

    # Example

    ```python
    import time
    print("Hello")

    with Timer():
            time.sleep(10)
    print("Done")

    ```
    output:

    "Hello"
    "Timer took 10 seconds"
    "Done"

    """

    def __init__(self, name=None):
        self.name = name

    def __enter__(self):
        self.start = time.time()
        return self

    def __exit__(self, *args):
        self.end = time.time()
        self.interval = self.end - self.start
        logger.info(f"Timer {self.name} took: {self.interval} seconds")



"""Vortexa Python SDK."""
# noinspection PyUnresolvedReferences
from vortexasdk.endpoints import (
    Attributes,
    CargoMovements,
    CargoTimeSeries,
    Corporations,
    Geographies,
    Products,
    VesselMovements,
    Vessels,
    EIAForecasts,
)

# noinspection PyUnresolvedReferences
from vortexasdk.version import __version__

# noinspection PyUnresolvedReferences
from vortexasdk.check_setup import run_all_checks



from typing import List, Union

from vortexasdk.endpoints.corporations import Corporations
from vortexasdk.api import ID
from vortexasdk.conversions.conversions import _convert_to_ids


def convert_to_corporation_ids(
    ids_or_names_list: List[Union[ID, str]]
) -> List[ID]:
    """
    Convert a mixed list of names or IDs to charterer ids.

    # Example
    ```
    >>> convert_to_corporation_ids(["Exon"])
    [...]

    ```
    """
    return _convert_to_ids(ids_or_names_list, Corporations())



from typing import Dict, TypeVar

import jsons

T = TypeVar("T")


def serialize_to_dict(dataclass) -> Dict:
    """Serialize data class attributes."""
    return jsons.loads(jsons.dumps(dataclass))


class FromDictMixin:
    """Mixin allowing serialization of a dictionary to a dataclass."""

    @classmethod
    def from_dict(cls: T, d: Dict) -> T:
        """Serialize dictionary to dataclass of type T."""
        return jsons.loads(jsons.dumps(d), cls)



from abc import ABC
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional, Union

from vortexasdk.api.id import ID

IDsNames = Union[List[Union[ID, str]], str, ID]

ISODate = str


# noinspection PyPep8Naming
def to_ISODate(utc_datetime: datetime) -> str:
    return utc_datetime.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "Z"


@dataclass(frozen=True)
class Entity:
    """Holds commonly used properties."""

    id: ID
    label: str
    layer: str


@dataclass(frozen=True)
class EntityWithProbability(Entity):
    """
    Extension of `Entity`, containing additional properties.

    - `probability` the probability of an entity occurring.
    - `source` the source of this entity, (is typically one of `['model', 'external_data']`
    """

    probability: float
    source: str


@dataclass(frozen=True)
class IDName:
    """Tuple containing `id` and `name`."""

    id: ID
    name: str


@dataclass(frozen=True)
class IDLayer:
    """Tuple containing `id` and `layer`."""

    id: ID
    layer: str


@dataclass(frozen=True)
class IDNameLayer:
    """Triple holding `id`, `name`, and `layer`."""

    id: ID
    layer: List[str]
    name: str


@dataclass(frozen=True)
class Node(ABC, IDName):
    """
    Abstract Base Class holding a node of a tree.

    # Attributes:
        ref_type: Identifies the reference type data
        leaf: Is this node a leaf of the hierarchical tree?
        parent: List of parents

    """

    ref_type: str
    leaf: bool
    parent: List[IDNameLayer]


@dataclass(frozen=True)
class Tag:
    """

    Represents a property that is associated with a period of time.

    A good example is if a vessel has acted as an FSO during a time period.

    [Tags Further Documentation](https://docs.vortexa.com/reference/intro-tags)

    """

    tag: str
    start_timestamp: Optional[ISODate] = None
    end_timestamp: Optional[ISODate] = None


@dataclass(frozen=True)
class Flag:
    """

    Represents a property that is associated with a vessel's flag.

    - `flag` key will be a Geography Entity ID.
    - `flag_country` key will be the ISO code for the country

    [Geography Entity Further Documentation](https://docs.vortexa.com/reference/intro-geography-entries)

    """

    tag: str
    flag: str
    flag_country: str


@dataclass(frozen=True)
class Scrubber:
    """

    Represents information about scrubbers fitted to a vessel.

    - `scrubber` key will be the type of scrubber.
    - `planned` key is if this scrubber has not yet been fitted but is planned.

    An empty `scrubber` List may mean the scrubber status is unknown or a vessel has none fitted.

    """

    tag: str
    scrubber: str
    planned: bool



from unittest import TestCase

import jsons

from vortexasdk.api.cargo_movement import CargoEvent
from vortexasdk.api.geography import GeographyEntity


class TestCargoEventEntity(TestCase):
    def test_serialize(self):
        with open("tests/api/examples/cargo_event_entity1.json", "r") as f:
            serialized = f.read()
            deserialized = jsons.loads(serialized, CargoEvent)

            expected = CargoEvent(
                event_type="cargo_port_unload_event",
                location=[
                    GeographyEntity(
                        id="2aaad41b89dfad19e5668918018ae02695d7710bcbe5f2dc689234e8da492de3",
                        layer="country",
                        label="United Kingdom",
                        source="model",
                        probability=1,
                    )
                ],
                probability=1,
                pos=[-0.256674902984994, 53.74191566386998],
                start_timestamp="2019-10-24T13:16:43+0000",
                end_timestamp="2019-10-25T00:40:46+0000",
            )

            assert expected == deserialized



from tests.testcases import TestCaseUsingMockAPI
from vortexasdk.api import Product
from vortexasdk.endpoints.products import Products


class TestProducts(TestCaseUsingMockAPI):
    def test_serialize(self):
        crude_dict = {
            "id": "6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653",
            "name": "Crude",
            "layer": ["group"],
            "leaf": False,
            "parent": [],
            "meta": {"api_min": 33.39586, "api_max": 33.39586},
            "ref_type": "product",
            "hierarchy": [
                {
                    "id": "6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653",
                    "layer": "group",
                    "label": "Crude",
                }
            ],
        }

        Product.from_dict(crude_dict)

    def test_search_ids_retreives_names(self):
        products = Products().search().to_df()
        assert len(products) > 0

    def test_to_list(self):
        Products().search().to_list()



from datetime import datetime

from docs.utils import to_markdown
from tests.testcases import TestCaseUsingRealAPI
from vortexasdk import Geographies, Products
from vortexasdk.endpoints.cargo_timeseries import CargoTimeSeries


class TestCargoTimeSeries(TestCaseUsingRealAPI):
    def test_search_returns_one_day(self):
        date = datetime(2019, 11, 10)

        result = CargoTimeSeries().search(
            filter_activity="loading_state",
            filter_time_min=date,
            filter_time_max=date,
        )

        assert len(result) == 1

    def test_search_returns_all_days(self):
        start = datetime(2019, 11, 1)
        end = datetime(2019, 11, 10)

        result = CargoTimeSeries().search(
            filter_activity="loading_state",
            filter_time_min=start,
            filter_time_max=end,
        )

        n_days = (end - start).days + 1

        assert n_days == len(result)

    def test_to_df(self):
        start = datetime(2019, 11, 1)
        end = datetime(2019, 11, 10)

        df = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                filter_time_min=start,
                filter_time_max=end,
            )
            .to_df()
        )

        print(to_markdown(df.head()))

        n_days = (end - start).days + 1

        assert len(df) == n_days
        assert list(df.columns) == ["key", "value", "count"]

    def test_to_list(self):
        start = datetime(2019, 11, 1)
        end = datetime(2019, 11, 10)

        time_series_list = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                filter_time_min=start,
                filter_time_max=end,
            )
            .to_list()
        )

        n_days = (end - start).days + 1

        assert len(time_series_list) == n_days

    def test_filter_geographies_and_products(self):
        start = datetime(2019, 1, 1)
        end = datetime(2019, 11, 1)

        rotterdam = [
            g.id
            for g in Geographies().search(term="rotterdam").to_list()
            if "port" in g.layer
        ]
        crude = [
            p.id
            for p in Products().search("crude").to_list()
            if "Crude" == p.name
        ]

        rotterdam_crude_timeseries = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                timeseries_unit="bpd",
                timeseries_frequency="month",
                filter_time_min=start,
                filter_time_max=end,
                filter_origins=rotterdam,
                filter_products=crude,
            )
            .to_df()
        )

        rotterdam_all_products_timeseries = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                timeseries_unit="bpd",
                timeseries_frequency="month",
                filter_time_min=start,
                filter_time_max=end,
                filter_origins=rotterdam,
            )
            .to_df()
        )

        assert (
            rotterdam_all_products_timeseries["value"].sum()
            > rotterdam_crude_timeseries["value"].sum()
        )

        print(rotterdam_crude_timeseries.head())

    def test_search_filters_on_timeseries_max_activity(self):
        df = (
            CargoTimeSeries()
            .search(
                filter_activity="storing_state",
                filter_time_min=datetime(2019, 8, 1),
                filter_time_max=datetime(2019, 8, 31),
                timeseries_activity_time_span_min=1000 * 60 * 60 * 24 * 14,
                timeseries_activity_time_span_max=1000 * 60 * 60 * 24 * 60,
            )
            .to_df()
            .head(2)
        )

        assert len(df) == 2

    def test_endpoint_respects_difference_between_timeseries_activity_and_filter_activity(
        self,
    ):
        min_ts = datetime(2019, 8, 1)
        max_ts = datetime(2019, 8, 31)

        unloading_activity = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                timeseries_activity="unloading_state",
                filter_time_min=min_ts,
                filter_time_max=max_ts,
            )
            .to_list()
        )

        loading_activity = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                timeseries_activity="loading_state",
                filter_time_min=min_ts,
                filter_time_max=max_ts,
            )
            .to_list()
        )

        assert loading_activity != unloading_activity

    def test_timeseries_activity_defaults_to_filter_activity(self):
        min_ts = datetime(2019, 8, 1)
        max_ts = datetime(2019, 8, 31)

        loading_activity_default = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                filter_time_min=min_ts,
                filter_time_max=max_ts,
            )
            .to_list()
        )

        loading_activity = (
            CargoTimeSeries()
            .search(
                filter_activity="loading_state",
                timeseries_activity="loading_state",
                filter_time_min=min_ts,
                filter_time_max=max_ts,
            )
            .to_list()
        )

        assert loading_activity == loading_activity_default



from datetime import datetime

from docs.utils import to_markdown
from tests.testcases import TestCaseUsingRealAPI
from tests.timer import Timer
from vortexasdk import Geographies, Corporations, Products
from vortexasdk.endpoints.cargo_movements import CargoMovements


class TestCargoMovementsReal(TestCaseUsingRealAPI):
    def test_default_search(self):
        results = CargoMovements().search(filter_activity="loading_state")
        print(len(results))

    def test_search_returns_unique_results(self):
        result = CargoMovements().search(
            filter_activity="loading_state",
            filter_origins=[
                "68faf65af1345067f11dc6723b8da32f00e304a6f33c000118fccd81947deb4e"
            ],
            filter_time_min=datetime(2017, 8, 29),
            filter_time_max=datetime(2017, 10, 29),
        )

        print("---------------------------------")
        n_results = len(result)
        print(f"Received {n_results} results")

        print(result[0])
        n_unique_results = len(set([str(k) for k in result]))
        print(f"Received {n_unique_results} unique results")

        assert n_results == n_unique_results

    def test_search_single_filter_id(self):
        df = (
            CargoMovements()
            .search(
                filter_activity="loading_state",
                filter_products="6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653",
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
            )
            .to_df()
            .head(2)
        )

        assert len(df) == 2

    def test_exlusion_filter(self):
        arab_medium = [
            p.id
            for p in Products().search("Arab Medium").to_list()
            if p.layer == ["grade"]
        ]
        meg = [
            g.id
            for g in Geographies().search("MEG/AG").to_list()
            if "trading_region" in g.layer
        ]
        iraq = [
            g.id
            for g in Geographies().search("Iraq").to_list()
            if "country" in g.layer
        ]

        cols = [
            "cargo_movement_id",
            "vessels.0.name",
            "events.cargo_port_load_event.0.location.port.id",
            "events.cargo_port_load_event.0.location.port.label",
            "events.cargo_port_load_event.0.location.country.id",
            "events.cargo_port_load_event.0.location.country.label",
            "events.cargo_port_load_event.0.start_timestamp",
            "events.cargo_port_load_event.0.end_timestamp",
            "product.grade.id",
            "product.grade.label",
            "product.group_product.label",
            "parent_ids.0.id",
            "parent_ids.0.splinter_timestamp"
        ]

        df = (
            CargoMovements()
            .search(
                filter_activity="loading_end",
                filter_products=[
                    "54af755a090118dcf9b0724c9a4e9f14745c26165385ffa7f1445bc768f06f11"
                ],
                filter_origins=meg,
                exclude_origins=iraq[0],
                exclude_products=arab_medium,
                filter_time_min=datetime(2019, 10, 1),
                filter_time_max=datetime(2019, 11, 1),
                cm_unit="b",
            )
            .to_df(columns=cols)
        )

        df_excl = df.loc[
            (
                df["events.cargo_port_load_event.0.location.country.id"]
                == iraq[0]
            )
            | (df["product.grade.id"] == arab_medium[0])
        ]

        assert df_excl.empty

    def test_to_df_all_columns(self):
        df = (
            CargoMovements()
            .search(
                filter_activity="loading_state",
                filter_products="6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653",
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
            )
            .to_df(columns="all")
            .head(2)
        )

        assert len(df) == 2

    def test_timestamp_columns(self):
        df = (
            CargoMovements()
            .search(
                filter_activity="loading_state",
                filter_products="6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653",
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
            )
            .to_df(columns="all")
            .head(2)
        )

        self.assertEqual(str(df['events.cargo_port_load_event.0.end_timestamp'].dtypes), 'datetime64[ns, UTC]')
        self.assertEqual(str(df['events.cargo_port_unload_event.0.end_timestamp'].dtypes), 'datetime64[ns, UTC]')

    def test_search_single_filter_origin_name(self):
        df = (
            CargoMovements()
            .search(
                filter_activity="loading_state",
                filter_origins=[
                    g.id
                    for g in Geographies().search(term="rotterdam").to_list()
                    if "port" in g.layer
                ],
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
            )
            .to_df()
            .head(2)
        )

        assert len(df) == 2

    def test_search_filters_on_timeseries_max_activity(self):
        df = (
            CargoMovements()
            .search(
                filter_activity="storing_state",
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
                timeseries_activity_time_span_min=1000 * 60 * 60 * 24 * 14,
                timeseries_activity_time_span_max=1000 * 60 * 60 * 24 * 60,
            )
            .to_df()
            .head(2)
        )

        assert len(df) == 2

    def test_search_single_filter_owner_name(self):
        df = (
            CargoMovements()
            .search(
                filter_activity="loading_state",
                filter_owners=[
                    c.id for c in Corporations().search(term="DHT").to_list()
                ],
                filter_time_min=datetime(2018, 10, 1, 0),
                filter_time_max=datetime(2018, 10, 5, 1),
            )
            .to_df()
            .head(2)
        )
        assert len(df) == 2

    def test_search_single_filter_waypoint_name(self):
        df = (
            CargoMovements()
            .search(
                filter_activity="any_activity",
                filter_waypoints=[
                    g.id for g in Geographies().search(term="suez").to_list()
                ],
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
            )
            .to_df()
            .head(2)
        )

        assert len(df) == 2

    def test_search_list_filter_id(self):
        df = (
            CargoMovements()
            .search(
                filter_products=[
                    "6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653"
                ],
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
                filter_activity="loading_state",
            )
            .to_df()
            .head(2)
        )

        assert len(df) == 2
        print(to_markdown(df))

    def test_search_to_list(self):
        CargoMovements().search(
            filter_products=[
                "6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653"
            ],
            filter_time_min=datetime(2019, 8, 29),
            filter_time_max=datetime(2019, 8, 29, 0, 10),
            filter_activity="loading_state",
        ).to_list()

    def test_speed(self):
        with Timer("Search") as t_search:
            cms = CargoMovements().search(
                filter_activity="loading_state",
                filter_time_min=datetime(2019, 8, 29),
                filter_time_max=datetime(2019, 8, 29, 0, 10),
            )

        with Timer("to_list") as t_to_list:
            cms.to_list()

        with Timer("df") as t_to_df:
            df = cms.to_df()

        # Check we load a reasonable number of cargo movements in a short enough period of time
        assert len(df) > 500
        assert t_search.interval < 10
        assert t_to_list.interval < 10
        assert t_to_df.interval < 5



from typing import List

import pandas as pd

from vortexasdk.api import EIAForecast
from vortexasdk.api.search_result import Result
from vortexasdk.logger import get_logger
from vortexasdk.result_conversions import create_dataframe, create_list

logger = get_logger(__name__)


class EIAForecastResult(Result):
    """Container class that holds the result obtained from calling the `EIAForecasts` endpoint."""

    def to_list(self) -> List[EIAForecast]:
        """Represent EIAForecast data as a list."""
        # noinspection PyTypeChecker
        return create_list(super().to_list(), EIAForecast)

    def to_df(self, columns=None) -> pd.DataFrame:
        """
        Represent EIA forecasts as a `pd.DataFrame`.

        # Arguments
            columns: The EIA forecasts columns we want in the dataframe. Enter `columns='all'` to include all columns.
            Defaults to `columns = ['date', 'forecast_fri', 'value', 'stocks', 'cover', 'runs']`.


        # Returns
        `pd.DataFrame` of EIA forecasts.
        """
        return create_dataframe(
            columns=columns,
            default_columns=DEFAULT_COLUMNS,
            data=super().to_list(),
            logger_description="EIAForecasts",
        )


DEFAULT_COLUMNS = ["date", "forecast_fri", "value", "stocks", "cover", "runs"]



import re
from typing import List, Tuple

ID = str


def is_valid_id(potential_id) -> bool:
    """Check if argument is valid SHA256 hash."""
    return (
        isinstance(potential_id, str)
        and re.match("^[A-Fa-f0-9]{64}$", potential_id) is not None
    )


def split_ids_other(ids_or_other: List) -> Tuple[List[ID], List]:
    """Extract IDs and names from a list, returning a list of IDs, and a list of names."""
    ids = []
    others = []

    for item in ids_or_other:
        if is_valid_id(item):
            ids.append(item)
        else:
            others.append(item)

    return ids, others



from tests.testcases import TestCaseUsingMockAPI
from vortexasdk.endpoints.vessels import Vessels


class TestVessels(TestCaseUsingMockAPI):
    def test_search_ids_retreives_names(self):
        vessels = Vessels().search().to_list()

        names = [x.name for x in vessels]

        assert names == ["0", "058"]

    def test_convert_to_df(self):
        df = Vessels().search().to_df()
        assert len(df) > 0



from dataclasses import dataclass
from typing import List, Optional

from vortexasdk.api.geography import GeographyEntity
from vortexasdk.api.product import ProductEntity
from vortexasdk.api.serdes import FromDictMixin
from vortexasdk.api.shared_types import ISODate
from vortexasdk.api.id import ID
from vortexasdk.api.vessel import VesselEntity


@dataclass(frozen=True)
class CargoEvent:
    """

    A CargoEvent represents an event that occurred to a cargo during a cargo movement.

    [Cargo Event Entities Further Documentation](https://docs.vortexa.com/reference/intro-cargo-events)

    """

    event_type: str
    location: List[GeographyEntity]

    probability: Optional[float] = None
    pos: Optional[List[float]] = None
    vessel_id: Optional[str] = None
    start_timestamp: Optional[ISODate] = None
    end_timestamp: Optional[ISODate] = None


@dataclass(frozen=True)
class ParentID:
    """

    `cargo_movement_id` may change under certain conditions. `ParentID` contains an `id`,
    a previous id of the cargo movement, and a `splinter_timestamp`, the time at which the id change occurred.

    [Cargo Movement Further Documentation](https://docs.vortexa.com/reference/intro-cargo-movement)

    """

    id: str
    splinter_timestamp: ISODate


@dataclass(frozen=True)
class CargoMovement(FromDictMixin):
    """

    Cargo movements are the base data set the Vortexa API is centred around.

    Each movement represents a journey of a certain quantity of a product between places.

    [Cargo Movement Further Documentation](https://docs.vortexa.com/reference/intro-cargo-movement)

    """

    cargo_movement_id: ID
    quantity: int
    status: str
    vessels: List[VesselEntity]
    product: List[ProductEntity]
    events: List[CargoEvent]
    parent_ids: List[ParentID]



from semver import VersionInfo

from vortexasdk.version import __version__
from vortexasdk.version_utils import get_latest_sdk_version


def check_proposed_version_is_allowed(
    latest_version: str, proposed_version: str
) -> None:
    """Check that the proposed version is a valid incremental semver version, or a prerelease."""

    latest = VersionInfo.parse(latest_version)
    proposed = VersionInfo.parse(proposed_version)

    next_versions = {
        latest.bump_major(),
        latest.bump_minor(),
        latest.bump_patch(),
    }

    proposed_version_is_allowed = (
        proposed in next_versions or proposed.prerelease is not None
    )

    if not proposed_version_is_allowed:
        raise Exception(
            f"The proposed version {proposed} is not allowed. \n"
            f"This might be because the proposed version already exists, \n"
            f"or because it isn't one of the next sequential semver version bumps. \n"
            f"The next version must be one of the following, or a pre-release: \n"
            f"{[str(v) for v in next_versions]}\n"
            f"You must change the vortexasdk/__version__.py file to a valid next version, following semver guidelines.\n"
            f"Refer to https://semver.org for more information on semantic versioning."
        )
    else:
        print(f"Latest version:  {latest_version}")
        print(f"Proposed version {proposed_version} is allowed.")


if __name__ == "__main__":
    check_proposed_version_is_allowed(
        latest_version=get_latest_sdk_version(), proposed_version=__version__
    )



from itertools import groupby
from typing import Dict, List

# noinspection PyProtectedMember
from flatten_dict import flatten
import copy


def flatten_dictionary(d: Dict) -> Dict:
    """Flatten dictionary, then format keys."""
    return _format_keys(flatten(d, enumerate_types=(list,)))


def _format_keys(dictionary):
    flat_with_formatted_keys = {}
    for k, v in dictionary.items():
        nice_path = ".".join([str(i) for i in k])
        flat_with_formatted_keys[nice_path] = v
    return flat_with_formatted_keys


def convert_cargo_movement_to_flat_dict(cme: Dict, cols="all") -> Dict:
    """Convert nested `CargoMovement` object to flat dictionary, keeping *cols*."""
    as_dict = _group_cargo_movement_attributes_by_layer(cme)

    formatted = flatten_dictionary(as_dict)

    if cols == "all":
        return formatted
    else:
        return {k: v for k, v in formatted.items() if k in cols}


def convert_vessel_movement_to_flat_dict(vm: Dict, cols="all") -> Dict:
    """Convert nested `VesselMovement` dict to flat dictionary, keeping *cols*."""
    as_dict = _group_vessel_movement_attributes_by_layer(vm)

    formatted = flatten_dictionary(as_dict)

    if cols == "all":
        return formatted
    else:
        return {k: v for k, v in formatted.items() if k in cols}


def _group_vessel_movement_attributes_by_layer(vm: Dict) -> Dict:
    """Group relevant `VesselMovement` attributes by `Entity.layer`."""
    if "origin" in vm.keys():
        flat_origin = _flatten_attributes(vm["origin"], "location")
        vm["origin"] = flat_origin

    if "destination" in vm.keys():
        flat_destination = _flatten_attributes(vm["destination"], "location")
        vm["destination"] = flat_destination

    if "cargoes" in vm.keys():
        flat_cargoes = [
            _flatten_attributes(cargo, "product") for cargo in vm["cargoes"]
        ]
        vm["cargoes"] = flat_cargoes

    vm["vessel"] = _flatten_vessel_entity(vm["vessel"])
    return vm


def _group_cargo_movement_attributes_by_layer(cm: Dict) -> Dict:
    """Group relevant `CargoMovement` attributes by `Entity.layer`."""
    vessels = [_flatten_vessel_entity(ve) for ve in cm["vessels"]]

    events = {
        event_type: list(g)
        for event_type, g in groupby(cm["events"], lambda x: x["event_type"])
    }

    events_attributes = {
        event_type: [_flatten_attributes(ce, "location") for ce in es]
        for event_type, es in events.items()
    }

    cm = _flatten_attributes(cm, "product")
    cm["vessels"] = vessels
    cm["events"] = events_attributes
    return cm


def _flatten_vessel_entity(vessel_entity: Dict) -> Dict:
    return _flatten_attributes(vessel_entity, "corporate_entities")


def _flatten_attributes(dictionary: Dict, key: str) -> Dict:
    """Group the key values by layer."""
    copied_dict = copy.deepcopy(dictionary)

    entity_list: List[Dict] = copied_dict[key]

    grouped_by_layer = {e["layer"]: e for e in entity_list}

    copied_dict[key] = grouped_by_layer

    return copied_dict


def _group_by_layer(entity_list: List[Dict]) -> Dict:
    return {e["layer"]: e for e in entity_list}



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

version = {}
with open("./vortexasdk/version.py") as fp:
    exec(fp.read(), version)

setuptools.setup(
    name="vortexasdk",
    version=version["__version__"],
    author="Vortexa Developers",
    author_email="developers@vortexa.com",
    description="Vortexa SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    url="https://github.com/vortechsa/python-sdk",
    license="Apache License 2.0",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=[
        "pandas>=0.25.2",
        "requests==2.22.0",
        "jsons==1.0.0",
        "flatten-dict==0.2.0",
        "tabulate==0.8.5",
        "tqdm==4.38.0",
    ],
    extras_require={
        "tests": [
            "mkdocs==1.0.4",
            "mypy==0.770",
            "pytest==5.2.4",
            "pre-commit==1.20.0",
            "flake8==3.7.9",
            "pydoc-markdown==2.0.5",
            "semver==2.13.0",
            "six==1.12.0",
            "xlrd==1.2.0"
        ],
        "deploy": [
            "wheel==0.36.2",
            "twine==3.3.0"
        ]
    },
)



"""Corporations Endpoint."""
from typing import Dict, List, Union

from vortexasdk.api import ID
from vortexasdk.endpoints.corporations_result import CorporationsResult
from vortexasdk.endpoints.endpoints import CORPORATIONS_REFERENCE
from vortexasdk.operations import Reference, Search
from vortexasdk.utils import convert_values_to_list


class Corporations(Reference, Search):
    """Corporations Endpoint."""

    def __init__(self):
        Reference.__init__(self, CORPORATIONS_REFERENCE)
        Search.__init__(self, CORPORATIONS_REFERENCE)

    def load_all(self) -> CorporationsResult:
        """Load all corporations."""
        return self.search()

    def search(
        self,
        term: Union[str, List[str]] = None,
        exact_term_match: bool = False,
    ) -> CorporationsResult:
        """
        Find all Corporations matching given search terms.

        # Arguments
            term: The corporation name(s) we're filtering on

             exact_term_match: Search on only exact term matches, or allow similar matches.
              e.g. When searching for "COS" with `exact_term_match=False`, then the SDK will yield corporations named
              ['COSCO', 'COSMO OIL'] etc. When `exact_term_match=True`,
              the SDK won't yield any results, because no corporations have the exact name "COS".

        # Returns
        List of corporation matching `term`


        # Examples

        Let's load all corporations
        ```python
        >>> from vortexasdk import Corporations
        >>> df = Corporations().search().to_df()

        ```
        returns

        |    | id                                                               | name       | corporate_entity_type   |
        |---:|:-----------------------------------------------------------------|:-----------|:------------------------|
        |  0 | 04f418ee78c1e17744ad653e7815e8e28891ed9ba25a8427030e4478e5c00974 | 3J         | ['commercial_owner']    |
        |  1 | b6384cf17f1639a64bbff04cfd32257bf732a3a13e4b0532802a9ae84a36be34 | 5XJAPANESE | ['commercial_owner']    |


        Let's find all corporations with 'do' in the name.
        ```python
        >>> [x.name for x in Corporations().search(term="do").to_list()]
        [...]

        ```

        # Further Documentation

        [VortexaAPI Corporation Reference](https://docs.vortexa.com/reference/POST/reference/charterers)

        """
        api_params = convert_values_to_list({"term": term})

        return CorporationsResult(
            super().search(exact_term_match=exact_term_match, **api_params)
        )

    def reference(self, id: ID) -> Dict:
        """
        Perform a corporation lookup.

        # Arguments
            id: Corporation ID to lookup

        # Returns
        Corporation record matching the ID

        # Further Documentation:
        [VortexaAPI Corporation Reference](https://docs.vortexa.com/reference/GET/reference/charterers/%7Bid%7D)

        # Examples
        >>> Corporations().reference(id='12345abcdef') # doctest: +SKIP

        """
        return super().reference(id)



from tests.testcases import TestCaseUsingMockAPI
from vortexasdk.endpoints.attributes import Attributes


class TestAttributes(TestCaseUsingMockAPI):
    def test_search_ids_retreives_names(self):
        attributes = Attributes().search().to_list()

        names = [x.name for x in attributes]

        assert names == ["Open Loop", "Unknown", "DFDE"]



from dataclasses import dataclass
from typing import List, Optional

from vortexasdk.api.geography import GeographyEntity
from vortexasdk.api.id import ID
from vortexasdk.api.serdes import FromDictMixin
from vortexasdk.api.shared_types import ISODate
from vortexasdk.api.vessel import VesselEntity


@dataclass(frozen=True)
class VesselEvent:
    """Represent an event that occurred to a vessel during a vessel movement."""

    event_type: str
    location: List[GeographyEntity]
    pos: Optional[List[float]] = None

    event_id: Optional[ID] = None
    start_timestamp: Optional[ISODate] = None
    end_timestamp: Optional[ISODate] = None


@dataclass(frozen=True)
class VesselMovement(FromDictMixin):
    """
    What is a vessel movement. TODO.

    [Vessel Movement Further Documentation](https://docs.vortexa.com/reference/intro-vessel-movement)

    """

    vessel_movement_id: ID
    voyage_id: ID
    vessel: VesselEntity

    origin: VesselEvent
    destination: VesselEvent
    start_timestamp: Optional[ISODate] = None
    end_timestamp: Optional[ISODate] = None



import functools
import os
from multiprocessing.pool import Pool
from typing import List

import pandas as pd

from vortexasdk.api import VesselMovement
from vortexasdk.api.entity_flattening import (
    convert_vessel_movement_to_flat_dict,
)
from vortexasdk.api.search_result import Result
from vortexasdk.result_conversions import create_dataframe, create_list
from vortexasdk.logger import get_logger

logger = get_logger(__name__)


class VesselMovementsResult(Result):
    """
    Container class holdings search results returns from the vessel movements endpoint.

    This class has two methods, `to_list()`, and `to_df()`, allowing search results to be represented as a list of `VesselMovement`s,
     or as a `pd.DataFrame` , respectively.
    """

    def to_list(self) -> List[VesselMovement]:
        """Represent vessel movements as a list of `VesselMovementEntity`s."""
        # noinspection PyTypeChecker
        return create_list(super().to_list(), VesselMovement)

    def to_df(self, columns=None) -> pd.DataFrame:
        """
        Represent vessel movements as a `pd.DataFrame`.

        # Arguments
            columns: Output columns present in the `pd.DataFrame`.
            Enter `columns='all'` to return all available columns.
            Enter `columns=None` to use `vessel_movements.DEFAULT_COLUMNS`.

        # Returns
        `pd.DataFrame`, one row per `VesselMovement`.


        ## Notes


        By default, the columns returned are something along the lines of.
        ```python
        DEFAULT_COLUMNS = [
            'vessel.imo',
            'vessel.name',
            'vessel.vessel_class',
            'origin.event_type',
            'origin.location.port.label',
            'origin.location.country.label',
            'destination.event_type',
            'destination.location.port.label',
            'destination.location.country.label',
            'cargoes.0.product.group_product.label,'
            'cargoes.0.product.category.label',
            'cargoes.0.product.grade.label',
            'cargoes.0.quantity',
            'start_timestamp',
            'end_timestamp',
        ]
        ```
        The exact default columns used can be found at `vessel_movements.DEFAULT_COLUMNS`

        A near complete list of columns is given below
        ```
        [
            cargoes.0.cargo_movement_id
            cargoes.0.product.category.id
            cargoes.0.product.category.label
            cargoes.0.product.category.layer
            cargoes.0.product.category.probability
            cargoes.0.product.category.source
            cargoes.0.product.grade.id
            cargoes.0.product.grade.label
            cargoes.0.product.grade.layer
            cargoes.0.product.grade.probability
            cargoes.0.product.grade.source
            cargoes.0.product.group.id
            cargoes.0.product.group.label
            cargoes.0.product.group.layer
            cargoes.0.product.group.probability
            cargoes.0.product.group.source
            cargoes.0.product.group_product.id
            cargoes.0.product.group_product.label
            cargoes.0.product.group_product.layer
            cargoes.0.product.group_product.probability
            cargoes.0.product.group_product.source
            cargoes.0.quantity
            cargoes.1.cargo_movement_id
            cargoes.1.product.category.id
            cargoes.1.product.category.label
            cargoes.1.product.category.layer
            cargoes.1.product.category.probability
            cargoes.1.product.category.source
            cargoes.1.product.grade.id
            cargoes.1.product.grade.label
            cargoes.1.product.grade.layer
            cargoes.1.product.grade.probability
            cargoes.1.product.grade.source
            cargoes.1.product.group.id
            cargoes.1.product.group.label
            cargoes.1.product.group.layer
            cargoes.1.product.group.probability
            cargoes.1.product.group.source
            cargoes.1.product.group_product.id
            cargoes.1.product.group_product.label
            cargoes.1.product.group_product.layer
            cargoes.1.product.group_product.probability
            cargoes.1.product.group_product.source
            cargoes.1.quantity
            cargoes.2.cargo_movement_id
            cargoes.2.product.category.id
            cargoes.2.product.category.label
            cargoes.2.product.category.layer
            cargoes.2.product.category.probability
            cargoes.2.product.category.source
            cargoes.2.product.grade.id
            cargoes.2.product.grade.label
            cargoes.2.product.grade.layer
            cargoes.2.product.grade.probability
            cargoes.2.product.grade.source
            cargoes.2.product.group.id
            cargoes.2.product.group.label
            cargoes.2.product.group.layer
            cargoes.2.product.group.probability
            cargoes.2.product.group.source
            cargoes.2.product.group_product.id
            cargoes.2.product.group_product.label
            cargoes.2.product.group_product.layer
            cargoes.2.product.group_product.probability
            cargoes.2.product.group_product.source
            cargoes.2.quantity
            cargoes.3.cargo_movement_id
            cargoes.3.product.category.id
            cargoes.3.product.category.label
            cargoes.3.product.category.layer
            cargoes.3.product.category.probability
            cargoes.3.product.category.source
            cargoes.3.product.grade.id
            cargoes.3.product.grade.label
            cargoes.3.product.grade.layer
            cargoes.3.product.grade.probability
            cargoes.3.product.grade.source
            cargoes.3.product.group.id
            cargoes.3.product.group.label
            cargoes.3.product.group.layer
            cargoes.3.product.group.probability
            cargoes.3.product.group.source
            cargoes.3.product.group_product.id
            cargoes.3.product.group_product.label
            cargoes.3.product.group_product.layer
            cargoes.3.product.group_product.probability
            cargoes.3.product.group_product.source
            cargoes.3.quantity
            cargoes.4.cargo_movement_id
            cargoes.4.product.category.id
            cargoes.4.product.category.label
            cargoes.4.product.category.layer
            cargoes.4.product.category.probability
            cargoes.4.product.category.source
            cargoes.4.product.grade.id
            cargoes.4.product.grade.label
            cargoes.4.product.grade.layer
            cargoes.4.product.grade.probability
            cargoes.4.product.grade.source
            cargoes.4.product.group.id
            cargoes.4.product.group.label
            cargoes.4.product.group.layer
            cargoes.4.product.group.probability
            cargoes.4.product.group.source
            cargoes.4.product.group_product.id
            cargoes.4.product.group_product.label
            cargoes.4.product.group_product.layer
            cargoes.4.product.group_product.probability
            cargoes.4.product.group_product.source
            cargoes.4.quantity
            cargoes.5.cargo_movement_id
            cargoes.5.product.category.id
            cargoes.5.product.category.label
            cargoes.5.product.category.layer
            cargoes.5.product.category.probability
            cargoes.5.product.category.source
            cargoes.5.product.grade.id
            cargoes.5.product.grade.label
            cargoes.5.product.grade.layer
            cargoes.5.product.grade.probability
            cargoes.5.product.grade.source
            cargoes.5.product.group.id
            cargoes.5.product.group.label
            cargoes.5.product.group.layer
            cargoes.5.product.group.probability
            cargoes.5.product.group.source
            cargoes.5.product.group_product.id
            cargoes.5.product.group_product.label
            cargoes.5.product.group_product.layer
            cargoes.5.product.group_product.probability
            cargoes.5.product.group_product.source
            cargoes.5.quantity
            cargoes.6.cargo_movement_id
            cargoes.6.product.category.id
            cargoes.6.product.category.label
            cargoes.6.product.category.layer
            cargoes.6.product.category.probability
            cargoes.6.product.category.source
            cargoes.6.product.grade.id
            cargoes.6.product.grade.label
            cargoes.6.product.grade.layer
            cargoes.6.product.grade.probability
            cargoes.6.product.grade.source
            cargoes.6.product.group.id
            cargoes.6.product.group.label
            cargoes.6.product.group.layer
            cargoes.6.product.group.probability
            cargoes.6.product.group.source
            cargoes.6.product.group_product.id
            cargoes.6.product.group_product.label
            cargoes.6.product.group_product.layer
            cargoes.6.product.group_product.probability
            cargoes.6.product.group_product.source
            cargoes.6.quantity
            cargoes.7.cargo_movement_id
            cargoes.7.product.category.id
            cargoes.7.product.category.label
            cargoes.7.product.category.layer
            cargoes.7.product.category.probability
            cargoes.7.product.category.source
            cargoes.7.product.grade.id
            cargoes.7.product.grade.label
            cargoes.7.product.grade.layer
            cargoes.7.product.grade.probability
            cargoes.7.product.grade.source
            cargoes.7.product.group.id
            cargoes.7.product.group.label
            cargoes.7.product.group.layer
            cargoes.7.product.group.probability
            cargoes.7.product.group.source
            cargoes.7.product.group_product.id
            cargoes.7.product.group_product.label
            cargoes.7.product.group_product.layer
            cargoes.7.product.group_product.probability
            cargoes.7.product.group_product.source
            cargoes.7.quantity
            cargoes.8.cargo_movement_id
            cargoes.8.product.category.id
            cargoes.8.product.category.label
            cargoes.8.product.category.layer
            cargoes.8.product.category.probability
            cargoes.8.product.category.source
            cargoes.8.product.grade.id
            cargoes.8.product.grade.label
            cargoes.8.product.grade.layer
            cargoes.8.product.grade.probability
            cargoes.8.product.grade.source
            cargoes.8.product.group.id
            cargoes.8.product.group.label
            cargoes.8.product.group.layer
            cargoes.8.product.group.probability
            cargoes.8.product.group.source
            cargoes.8.product.group_product.id
            cargoes.8.product.group_product.label
            cargoes.8.product.group_product.layer
            cargoes.8.product.group_product.probability
            cargoes.8.product.group_product.source
            cargoes.8.quantity
            cargoes.9.cargo_movement_id
            cargoes.9.product.category.id
            cargoes.9.product.category.label
            cargoes.9.product.category.layer
            cargoes.9.product.category.probability
            cargoes.9.product.category.source
            cargoes.9.product.grade.id
            cargoes.9.product.grade.label
            cargoes.9.product.grade.layer
            cargoes.9.product.grade.probability
            cargoes.9.product.grade.source
            cargoes.9.product.group.id
            cargoes.9.product.group.label
            cargoes.9.product.group.layer
            cargoes.9.product.group.probability
            cargoes.9.product.group.source
            cargoes.9.product.group_product.id
            cargoes.9.product.group_product.label
            cargoes.9.product.group_product.layer
            cargoes.9.product.group_product.probability
            cargoes.9.product.group_product.source
            cargoes.9.quantity
            destination.end_timestamp
            destination.event_id
            destination.event_type
            destination.from_vessel.id
            destination.from_vessel.label
            destination.from_vessel.tags.0
            destination.from_vessel.tags.1
            destination.location.country.id
            destination.location.country.label
            destination.location.country.layer
            destination.location.country.probability
            destination.location.country.source
            destination.location.port.id
            destination.location.port.label
            destination.location.port.layer
            destination.location.port.probability
            destination.location.port.source
            destination.location.region.id
            destination.location.region.label
            destination.location.region.layer
            destination.location.region.probability
            destination.location.region.source
            destination.location.shipping_region.id
            destination.location.shipping_region.label
            destination.location.shipping_region.layer
            destination.location.shipping_region.probability
            destination.location.shipping_region.source
            destination.location.sts_zone.id
            destination.location.sts_zone.label
            destination.location.sts_zone.layer
            destination.location.sts_zone.probability
            destination.location.sts_zone.source
            destination.location.terminal.id
            destination.location.terminal.label
            destination.location.terminal.layer
            destination.location.terminal.probability
            destination.location.terminal.source
            destination.location.trading_block.id
            destination.location.trading_block.label
            destination.location.trading_block.layer
            destination.location.trading_block.probability
            destination.location.trading_block.source
            destination.location.trading_region.id
            destination.location.trading_region.label
            destination.location.trading_region.layer
            destination.location.trading_region.probability
            destination.location.trading_region.source
            destination.location.trading_subregion.id
            destination.location.trading_subregion.label
            destination.location.trading_subregion.layer
            destination.location.trading_subregion.probability
            destination.location.trading_subregion.source
            destination.pos.0
            destination.pos.1
            destination.start_timestamp
            destination.to_vessel.id
            destination.to_vessel.label
            destination.to_vessel.tags.0
            destination.to_vessel.tags.1
            destination.to_vessel.tags.2
            destination.to_vessel.tags.3
            destination.to_vessel.tags.4
            destination.to_vessel.tags.5
            destination.to_vessel.tags.6
            destination.to_vessel.tags.7
            destination.to_vessel.tags.8
            destination.to_vessel.tags.9
            end_timestamp
            origin.end_timestamp
            origin.event_id
            origin.event_type
            origin.from_vessel.id
            origin.from_vessel.label
            origin.from_vessel.tags.0
            origin.from_vessel.tags.1
            origin.from_vessel.tags.2
            origin.from_vessel.tags.3
            origin.from_vessel.tags.4
            origin.from_vessel.tags.5
            origin.location.country.id
            origin.location.country.label
            origin.location.country.layer
            origin.location.country.probability
            origin.location.country.source
            origin.location.port.id
            origin.location.port.label
            origin.location.port.layer
            origin.location.port.probability
            origin.location.port.source
            origin.location.region.id
            origin.location.region.label
            origin.location.region.layer
            origin.location.region.probability
            origin.location.region.source
            origin.location.shipping_region.id
            origin.location.shipping_region.label
            origin.location.shipping_region.layer
            origin.location.shipping_region.probability
            origin.location.shipping_region.source
            origin.location.sts_zone.id
            origin.location.sts_zone.label
            origin.location.sts_zone.layer
            origin.location.sts_zone.probability
            origin.location.sts_zone.source
            origin.location.terminal.id
            origin.location.terminal.label
            origin.location.terminal.layer
            origin.location.terminal.probability
            origin.location.terminal.source
            origin.location.trading_block.id
            origin.location.trading_block.label
            origin.location.trading_block.layer
            origin.location.trading_block.probability
            origin.location.trading_block.source
            origin.location.trading_region.id
            origin.location.trading_region.label
            origin.location.trading_region.layer
            origin.location.trading_region.probability
            origin.location.trading_region.source
            origin.location.trading_subregion.id
            origin.location.trading_subregion.label
            origin.location.trading_subregion.layer
            origin.location.trading_subregion.probability
            origin.location.trading_subregion.source
            origin.pos.0
            origin.pos.1
            origin.start_timestamp
            origin.to_vessel.id
            origin.to_vessel.label
            origin.to_vessel.tags.0
            origin.to_vessel.tags.1
            origin.to_vessel.tags.2
            origin.to_vessel.tags.3
            start_timestamp
            vessel.corporate_entities.charterer.id
            vessel.corporate_entities.charterer.label
            vessel.corporate_entities.charterer.layer
            vessel.corporate_entities.charterer.probability
            vessel.corporate_entities.charterer.source
            vessel.corporate_entities.commercial_owner.id
            vessel.corporate_entities.commercial_owner.label
            vessel.corporate_entities.commercial_owner.layer
            vessel.corporate_entities.commercial_owner.probability
            vessel.corporate_entities.commercial_owner.source
            vessel.corporate_entities.time_charterer.end_timestamp
            vessel.corporate_entities.time_charterer.id
            vessel.corporate_entities.time_charterer.label
            vessel.corporate_entities.time_charterer.layer
            vessel.corporate_entities.time_charterer.probability
            vessel.corporate_entities.time_charterer.source
            vessel.corporate_entities.time_charterer.start_timestamp
            vessel.cubic_capacity
            vessel.dwt
            vessel.id
            vessel.imo
            vessel.mmsi
            vessel.name
            vessel.status
            vessel.tags.0.end_timestamp
            vessel.tags.0.start_timestamp
            vessel.tags.0.tag
            vessel.tags.1.end_timestamp
            vessel.tags.1.start_timestamp
            vessel.tags.1.tag
            vessel.tags.2.end_timestamp
            vessel.tags.2.start_timestamp
            vessel.tags.2.tag
            vessel.tags.3.end_timestamp
            vessel.tags.3.start_timestamp
            vessel.tags.3.tag
            vessel.tags.4.end_timestamp
            vessel.tags.4.start_timestamp
            vessel.tags.4.tag
            vessel.vessel_class
            vessel_movement_id
            voyage_id
        ]
        ```

        """
        if columns is None:
            columns = DEFAULT_COLUMNS

        logger.debug("Converting each VesselMovement to a flat dictionary")
        flatten = functools.partial(
            convert_vessel_movement_to_flat_dict, cols=columns
        )

        with Pool(os.cpu_count()) as pool:
            records = pool.map(flatten, super().to_list())

        return create_dataframe(
            columns=columns,
            default_columns=DEFAULT_COLUMNS,
            data=records,
            logger_description="VesselMovements",
        )


DEFAULT_COLUMNS = [
    "vessel.name",
    "vessel.imo",
    "vessel.mmsi",
    "vessel.cubic_capacity",
    "vessel.dwt",
    "vessel.vessel_class",
    "origin.location.port.label",
    "origin.location.sts_zone.label",
    "origin.from_vessel.label",
    "origin.to_vessel.label",
    "destination.location.port.label",
    "destination.location.sts_zone.label",
    "destination.from_vessel.label",
    "destination.to_vessel.label",
    "origin.start_timestamp",
    "destination.end_timestamp",
    "cargoes.0.product.group.label",
    "cargoes.0.product.grade.label",
    "cargoes.0.product.grade.probability",
    "vessel.corporate_entities.charterer.label",
    "vessel.corporate_entities.time_charterer.label",
    "vessel.corporate_entities.commercial_owner.label",
]



import os
import pathlib

import six


# From https://github.com/keras-team/keras/blob/0a0ac3fa5462cf4a72636ca4498a0a82ac91fc32/docs/autogen.py


def get_module_docstring(filepath):
    """Extract the module docstring.
    Also finds the line at which the docstring ends.
    """
    co = compile(open(filepath, encoding="utf-8").read(), filepath, "exec")
    if co.co_consts and isinstance(co.co_consts[0], six.string_types):
        docstring = co.co_consts[0]
    else:
        print("Could not get the docstring from " + filepath)
        docstring = ""
    return docstring, co.co_firstlineno


def copy_examples(examples_dir, destination_dir):
    """Copy the examples directory in the documentation.
    Prettify files by extracting the docstrings written in Markdown.
    """
    pathlib.Path(destination_dir).mkdir(exist_ok=True)
    for file in os.listdir(examples_dir):
        if not file.endswith(".py"):
            continue
        module_path = os.path.join(examples_dir, file)
        docstring, starting_line = get_module_docstring(module_path)
        destination_file = os.path.join(destination_dir, file[:-2] + "md")
        with open(destination_file, "w+", encoding="utf-8") as f_out, open(
            os.path.join(examples_dir, file), "r+", encoding="utf-8"
        ) as f_in:

            f_out.write(docstring + "\n\n")

            # skip docstring
            for _ in range(starting_line):
                next(f_in)

            f_out.write("```python\n")
            # next line might be empty.
            line = next(f_in)
            if line != "\n":
                f_out.write(line)

            # copy the rest of the file.
            for line in f_in:
                f_out.write(line)
            f_out.write("```")


if __name__ == "__main__":
    print(os.getcwd())
    copy_examples("./docs/examples", "./_build/pydocmd/examples")



from tests.testcases import TestCaseUsingRealAPI
from vortexasdk import Geographies
from vortexasdk.conversions import (
    convert_to_corporation_ids,
    convert_to_geography_ids,
    convert_to_product_ids,
)


class TestConvert(TestCaseUsingRealAPI):
    def test_convert_to_geography_ids(self):
        rotterdam_id = (
            "68faf65af1345067f11dc6723b8da32f00e304a6f33c000118fccd81947deb4e"
        )
        rotterdam_name = Geographies().reference(rotterdam_id)["name"]

        result = convert_to_geography_ids([rotterdam_name])

        assert [rotterdam_id] == result

    def test_convert_to_corporation_ids(self):
        result = convert_to_corporation_ids(["DHT"])

        assert len(result) >= 1

    def test_convert_to_product_ids(self):
        result = convert_to_product_ids(["crude"])

        CRUDE_ID = (
            "6f11b0724c9a4e85ffa7f1445bc768f054af755a090118dcf99f14745c261653"
        )
        assert CRUDE_ID in result

    def test_convert_to_ids_retains_id_argument(self):
        id = "f00e3068faf65af1345067f11dc6723b8da324a6f33c000118fccd81947deb4e"
        corporation_result = convert_to_corporation_ids(["DHT"])
        corporation_result_with_id = convert_to_corporation_ids(["DHT", id])

        assert corporation_result_with_id == [id] + corporation_result



from typing import List, Union

from vortexasdk.api import ID
from vortexasdk.api.id import split_ids_other
from vortexasdk.conversions.conversions import _search_ids
from vortexasdk.endpoints.vessels import AVAILABLE_VESSEL_CLASSES, Vessels
from vortexasdk.utils import convert_to_list


def convert_to_vessel_ids(
    vessel_attributes: Union[List[Union[ID, str, int]], ID, str, int]
) -> List[ID]:
    """
    Convert a mixed list of names, IDs, IMOs, or MMSIs to vessel ids.

    # Example
    ```
    >>> convert_to_vessel_ids(["Stallion", 9464326, 477639900, 'vlcc'])
    [...]

    ```
    """

    vessel_attributes_list = convert_to_list(vessel_attributes)
    ids, others = split_ids_other(vessel_attributes_list)

    vessel_classes = []
    names_imos_mmsis = []
    for e in others:
        if _is_vessel_class(e):
            vessel_classes.append(e)
        else:
            names_imos_mmsis.append(e)

    vessels_matched_on_vessel_class = (
        _search_ids(Vessels(), vessel_classes=vessel_classes)
        if len(vessel_classes) > 0
        else []
    )

    vessels_matched_on_name_imo_mmsi = (
        _search_ids(Vessels(), term=names_imos_mmsis)
        if len(names_imos_mmsis) > 0
        else []
    )

    return (
        ids
        + vessels_matched_on_vessel_class
        + vessels_matched_on_name_imo_mmsi
    )


def _is_vessel_class(potential_vessel_class) -> bool:
    return (
        isinstance(potential_vessel_class, str)
        and potential_vessel_class.lower() in AVAILABLE_VESSEL_CLASSES
    )



from unittest import TestCase

import pandas as pd

from tests.mock_client import example_vessels
from vortexasdk.endpoints.vessels_result import VesselsResult


class TestVesselsSearchResult(TestCase):
    vsr = VesselsResult(example_vessels)

    def test_to_df(self):
        df = self.vsr.to_df("all")

        expected = pd.DataFrame(
            {
                "call_sign": {0: "3269", 1: "839"},
                "dead_weight": {0: 6403, 1: 5000},
                "id": {
                    0: "1f6145c6e8134bed20a48d5a2a77732ffb2fe4e679b4297b25582dcb93dfaa84",
                    1: "1761da4fb069cd6ce153b6ad1c48e15cdb994eb386e4aafbe8f1bbde993cbaef",
                },
                "mmsi": {0: 900403269, 1: 413771781},
                "name": {0: "0", 1: "058"},
                "to_bow": {0: 120, 1: 18},
                "to_port": {0: 30, 1: 47},
                "to_starboard": {0: 5, 1: 36},
                "to_stern": {0: 5, 1: 28},
                "related_names": {0: ["0"], 1: ["058"]},
                "current_product_type": {
                    0: [
                        {
                            "cargo_movement_id": "12d2cbca522e5570ea988b30850ad5b762e16d1b710e920f26729f08fcfdca46",
                            "product_type": [
                                {
                                    "id": "b68cbb746f8b9098c50e2ba36bcad83001a53bd362e9031fb49085d02c36659c",
                                    "layer": "group",
                                    "label": "Clean products",
                                },
                                {
                                    "id": "cbb5acffa542aad31d26712a8cdfd9884301aa7658ac95c40f86ddb31bc29d9b",
                                    "layer": "group_product",
                                    "label": "Other Clean products",
                                },
                            ],
                            "active": False,
                        }
                    ],
                    1: [
                        {
                            "cargo_movement_id": "5b74c0aab0353cccb28a1bed34c04d7fa54120efb32de10f8f0429c00dd8d536",
                            "product_type": [
                                {
                                    "id": "b68cbb746f8b9098c50e2ba36bcad83001a53bd362e9031fb49085d02c36659c",
                                    "layer": "group",
                                    "label": "Clean products",
                                },
                                {
                                    "id": "cbb5acffa542aad31d26712a8cdfd9884301aa7658ac95c40f86ddb31bc29d9b",
                                    "layer": "group_product",
                                    "label": "Other Clean products",
                                },
                            ],
                            "active": False,
                        },
                        {
                            "cargo_movement_id": "8709d0cf0d103349c6df89f4b53ac90babc89e53c966f8cbf42aa0e93be20d96",
                            "product_type": [
                                {
                                    "id": "b68cbb746f8b9098c50e2ba36bcad83001a53bd362e9031fb49085d02c36659c",
                                    "layer": "group",
                                    "label": "Clean products",
                                },
                                {
                                    "id": "a75fcc09bfc7d16496de3336551bc52b5891838bb7c22356d2cb65451587d1e5",
                                    "layer": "group_product",
                                    "label": "Biodiesel",
                                },
                            ],
                            "active": False,
                        },
                    ],
                },
                "parent": {
                    0: [
                        {
                            "name": "Tiny Tanker",
                            "layer": ["vessel_class"],
                            "id": "tiny_tanker",
                        }
                    ],
                    1: [
                        {
                            "name": "Tiny Tanker",
                            "layer": ["vessel_class"],
                            "id": "tiny_tanker",
                        }
                    ],
                },
                "ref_type": {0: "vessel", 1: "vessel"},
                "vessel_status": {
                    0: "vessel_status_ballast",
                    1: "vessel_status_laden_unknown",
                },
                "corporate_entities": {0: [], 1: []},
                "leaf": {0: True, 1: True},
                "vessel_class": {0: "tiny_tanker", 1: "tiny_tanker"},
                "cubic_capacity": {0: 7060, 1: 6318},
                "tags": {
                    0: [
                        {
                            "tag": "vessel_decommissioned_tag",
                            "start_timestamp": "2016-06-08T09:44:57+0000",
                        }
                    ],
                    1: [
                        {
                            "tag": "vessel_decommissioned_tag",
                            "start_timestamp": "2017-07-11T13:42:37+0000",
                        }
                    ],
                },
                "layer": {0: ["vessel"], 1: ["vessel"]},
            }
        )

        assert expected.equals(df)



from typing import Dict, List

from vortexasdk.api.id import ID
from vortexasdk.client import default_client
from vortexasdk.exceptions import InvalidAPIDataResponseException
from vortexasdk.logger import get_logger
from vortexasdk.utils import filter_exact_match

logger = get_logger(__name__)


class Reference:
    """Lookup Vortexa Reference Data using an entity ID."""

    def __init__(self, resource):
        """
        Init.

        # Arguments
            resource: The vortexa endpoint used for reference lookups.

        """
        self._resource = resource

    def reference(self, id: ID) -> Dict:
        """
        Lookup reference data using ID.

        # Arguments
            id: ID of the entity we're looking up

        # Returns
        An entity matching the ID

        # Examples

        >>> Reference("/reference/geographies").reference(id='cfb8c4ef76585c3a37792b643791a0f4ff6d5656d5508927d8017319e21f2fca') # doctest: +SKIP

        """
        logger.info(
            f"Looking up {self.__class__.__name__} reference data with id: {id}"
        )

        data = default_client().get_reference(self._resource, id)

        assert len(data) <= 1, InvalidAPIDataResponseException(
            f"Server error: more than one record returned matching ID {id}"
        )
        try:
            return data[0]
        except IndexError:
            return {}


class Search:
    """Search Vortexa Reference Data."""

    def __init__(self, resource):
        """
        Init.

        # Arguments
            resource: Appropriate search resource

        """
        self._resource = resource

    def search(
        self, exact_term_match: bool = None, **api_params
    ) -> List[dict]:
        """
        Search Reference data filtering on `params`.

        # Arguments
            exact_term_match: Optional argument to filter names on exact matches
            api_params: Search parameters to be passed on to the API

        # Returns
        Result of VortexaAPI call from hitting querying the `resource` endpoint filtering with `params`.

        # Examples

        >>> Search("/reference/vessels").search(term="DHT") # doctest: +SKIP

        """
        logger.info(f"Searching {self.__class__.__name__}")
        api_result = default_client().search(self._resource, **api_params)
        logger.debug(
            f"{len(api_result)} results received from {self._resource}"
        )

        if exact_term_match:
            logger.debug("Filtering results on exact term match")
            return filter_exact_match(api_params["term"], api_result)
        else:
            return api_result



"""Time Series Endpoint."""
from datetime import datetime
from typing import List, Union

from vortexasdk.api.shared_types import to_ISODate
from vortexasdk.endpoints.endpoints import CARGO_TIMESERIES_RESOURCE
from vortexasdk.endpoints.timeseries_result import TimeSeriesResult
from vortexasdk.operations import Search
from vortexasdk.utils import convert_to_list


class CargoTimeSeries(Search):
    _MAX_PAGE_RESULT_SIZE = 500

    def __init__(self):
        Search.__init__(self, CARGO_TIMESERIES_RESOURCE)

    # noinspection PyUnresolvedReferences
    def search(
        self,
        filter_activity: str,
        timeseries_activity: str = None,
        timeseries_frequency: str = "day",
        timeseries_unit: str = "b",
        filter_time_min: datetime = datetime(2019, 10, 1, 0),
        filter_time_max: datetime = datetime(2019, 10, 1, 1),
        filter_charterers: Union[str, List[str]] = None,
        filter_destinations: Union[str, List[str]] = None,
        filter_origins: Union[str, List[str]] = None,
        filter_owners: Union[str, List[str]] = None,
        filter_products: Union[str, List[str]] = None,
        filter_vessels: Union[str, List[str]] = None,
        filter_storage_locations: Union[str, List[str]] = None,
        filter_ship_to_ship_locations: Union[str, List[str]] = None,
        filter_waypoints: Union[str, List[str]] = None,
        disable_geographic_exclusion_rules: bool = None,
        timeseries_activity_time_span_min: int = None,
        timeseries_activity_time_span_max: int = None,
    ) -> TimeSeriesResult:
        """

        Find Aggregate flows between regions, for various products, for various vessels, or various corporations.

        Example questions that can be answered with this endpoint:

        * _How many Crude/Condensate barrels have been imported into China each day over the last year?_
        * _How many tonnes of Fuel Oil has company X exported from the United States each week over the last 2 years?_
        * _How have long-term Medium-Sour floating storage levels changed over time?_

        # Arguments
            filter_activity: Cargo movement activity on which to base the time filter. The endpoint only includes cargo
            movements matching that match this filter in the aggregations. Must be one of ['loading_state',
             'loading_start', 'loading_end', 'identified_for_loading_state', 'unloading_state', 'unloading_start',
              'unloading_end', 'storing_state', 'storing_start', 'storing_end', 'transiting_state'].

            filter_time_min: The UTC start date of the time filter.

            filter_time_max: The UTC end date of the time filter.

            filter_corporations: A corporation ID, or list of corporation IDs to filter on.

            filter_destinations: A geography ID, or list of geography IDs to filter on.

            filter_origins: A geography ID, or list of geography IDs to filter on.

            filter_owners: An owner ID, or list of owner IDs to filter on.

            filter_products: A product ID, or list of product IDs to filter on.

            filter_vessels: A vessel ID, or list of vessel IDs to filter on.

            filter_storage_locations: A geography ID, or list of geography IDs to filter on.

            filter_ship_to_ship_locations: A geography ID, or list of geography IDs to filter on.

            filter_waypoints: A geography ID, or list of geography IDs to filter on.

            disable_geographic_exclusion_rules: This controls a popular industry term "intra-movements" and determines
             the filter behaviour for cargo leaving then entering the same geographic area.

            timeseries_activity: The cargo movement activity we want to aggregate on. This param defaults to
            `filter_activity` if left blank. For example, Let's say we want to aggregate the unloading timestamps of
             all cargo movements that loaded in 2019, then we'd use `filter_time_min` and `filter_time_max` to specify
             1st Jan 2019 and 31st Dec 2019 respectively, we'd set `filter_activity='loading_state'` and
             `timeseries_activity='unloading_state'` to filter on loadings but aggregate on unloadings.
              `filter_activity` Must be one of ['loading_state',
             'loading_start', 'loading_end', 'identified_for_loading_state', 'unloading_state', 'unloading_start',
              'unloading_end', 'storing_state', 'storing_start', 'storing_end', 'transiting_state'].

            timeseries_frequency: Frequency denoting the granularity of the time series. Must be one of ['day', 'week',
             'doe_week', 'month', 'quarter', 'year']

            timeseries_unit: A numeric metric to be calculated for each time bucket. Must be one of ['b', 'bpd', 't',
             'tpd', 'c', 'cpd'], corresponding to barrels, barrels per day, metric tonnes, metric tonnes per day,
              cargo movement count, cargo movement count per day, respectively.

            timeseries_activity_time_span_min: The minimum amount of time in milliseconds accounted for in a time series
             activity. Can be used to request long-term floating storage. For example, to only return floating storage
             movements that occurred for _more_ than 14 days enter
             `timeseries_activity_time_span_min=1000 * 60 * 60 * 24 * 14` in conjunction with
             `filter_activity='storing_state'`.

            timeseries_activity_time_span_max: The maximum amount of time in milliseconds accounted for in a time series
             activity. Can be used to request short-term floating storage. For example, to only return floating storage
             movements that occurred for _less_ than 14 days enter
             `timeseries_activity_time_span_max=1000 * 60 * 60 * 24 * 14` in conjunction with
             `filter_activity='storing_state'`.

        # Returns
        `TimeSeriesResult`

        # Example

        * _What was the monthly average barrels per day of crude loaded from Rotterdam over the last year?_

        ```python
        >>> from vortexasdk import CargoTimeSeries, Geographies, Products
        >>> rotterdam = [g.id for g in Geographies().search("rotterdam").to_list() if "port" in g.layer]
        >>> crude = [p.id for p in Products().search("crude").to_list() if "Crude" == p.name]
        >>> search_result = CargoTimeSeries().search(
        ...    timeseries_unit='bpd',
        ...    timeseries_frequency='month',
        ...    filter_origins=rotterdam,
        ...    filter_products=crude,
        ...    filter_activity='loading_state',
        ...    filter_time_min=datetime(2018, 1, 1),
        ...    filter_time_max=datetime(2018, 12, 31))
        >>> df = search_result.to_df()

        ```

        Gives the following:

        |    | key                      |     count |     value |
        |---:|:-------------------------|----------:|----------:|
        |  0 | 2018-01-01T00:00:00.000Z | 0.354839  | 458665    |
        |  1 | 2018-02-01T00:00:00.000Z | 0.75      | 45024     |
        |  2 | 2018-03-01T00:00:00.000Z | 0.0645161 |  35663.5  |
        |  3 | 2018-04-01T00:00:00.000Z | 0.878777  |  12345.2  |
        |  4 | 2018-05-01T00:00:00.000Z | 0.455932  |   9999.32 |
        |  5 | 2018-06-01T00:00:00.000Z | 0.777667  |  12234.8  |
        |  6 | 2018-07-01T00:00:00.000Z | 0.555097  | 987666    |
        |  7 | 2018-08-01T00:00:00.000Z | 0.290323  | 5318008.1 |
        |  8 | 2018-09-01T00:00:00.000Z | 0.0333333 | 686888.87 |
        |  9 | 2018-10-01T00:00:00.000Z | 0.354839  | 234344    |
        | 10 | 2018-11-01T00:00:00.000Z | 0.2345    | 111111    |
        | 11 | 2018-12-01T00:00:00.000Z | 0.123129  |  34344.5  |


        """
        api_params = {
            "filter_activity": filter_activity,
            "filter_time_min": to_ISODate(filter_time_min),
            "filter_time_max": to_ISODate(filter_time_max),
            "timeseries_activity_time_span_min": timeseries_activity_time_span_min,
            "timeseries_activity_time_span_max": timeseries_activity_time_span_max,
            "filter_charterers": convert_to_list(filter_charterers),
            "filter_owners": convert_to_list(filter_owners),
            "filter_products": convert_to_list(filter_products),
            "filter_vessels": convert_to_list(filter_vessels),
            "filter_destinations": convert_to_list(filter_destinations),
            "filter_origins": convert_to_list(filter_origins),
            "filter_storage_locations": convert_to_list(
                filter_storage_locations
            ),
            "filter_ship_to_ship_locations": convert_to_list(
                filter_ship_to_ship_locations
            ),
            "filter_waypoints": convert_to_list(filter_waypoints),
            "disable_geographic_exclusion_rules": disable_geographic_exclusion_rules,
            "timeseries_frequency": timeseries_frequency,
            "timeseries_unit": timeseries_unit,
            "timeseries_activity": timeseries_activity or filter_activity,
            "size": self._MAX_PAGE_RESULT_SIZE,
        }

        return TimeSeriesResult(super().search(**api_params))



from unittest import TestCase

import jsons

from vortexasdk.api.vessel import VesselEntity
from vortexasdk.api.corporation import CorporateEntity

ce1 = CorporateEntity(
    id="cbd7dfe8a9fb0fa0ce3252ce7643437db6a32d0947a0c23d68dc5dea2f2d65d7",
    layer="commercial_owner",
    probability=1,
    label="NGM Energy",
    source="external",
)

ce2 = CorporateEntity(
    id="0bdf9acdc00ad52d9b8c44dad815087a89205a9c83f53ed89e029f7d15b9ac14",
    layer="charterer",
    probability=1,
    label="CHEVRON",
    source="external",
)
ve = VesselEntity(
    id="1f6d6ab80d90846cedddf1a4b470faf566fd72595e853bc769185fac6ae7af70",
    mmsi=403534000,
    imo=9779836,
    name="RIMTHAN",
    dwt=298855,
    cubic_capacity=327310,
    vessel_class="vlcc_plus",
    corporate_entities=[ce1, ce2],
    start_timestamp="2019-10-14T00:00:00+0000",
    fixture_id="cc4f364367db830da1d5d8ad89b02a7dc6f207402cd907a19c9d713e1a70b0ed",
    fixture_fulfilled=False,
    voyage_id="813cb0baf01a335a5c74aded99b5c910bf627b24ff7b5eb9ed943579e902495d",
    tags=[],
    status="vessel_status_laden_known",
)


class TestVesselEntity(TestCase):
    def test_serialize(self):
        with open("tests/api/examples/vessel_entity1.json", "r") as f:
            serialized = f.read()
            deserialized = jsons.loads(serialized, VesselEntity)

            assert ve == deserialized



"""Vortexa API."""
from vortexasdk.api.attribute import Attribute
from vortexasdk.api.cargo_movement import CargoEvent, CargoMovement, ParentID
from vortexasdk.api.corporation import CorporateEntity, Corporation
from vortexasdk.api.geography import (
    BoundingBox,
    Geography,
    GeographyEntity,
    Position,
)
from vortexasdk.api.id import ID
from vortexasdk.api.product import Product, ProductEntity
from vortexasdk.api.shared_types import (
    Entity,
    EntityWithProbability,
    IDName,
    IDNameLayer,
    ISODate,
)
from vortexasdk.api.vessel import Vessel, VesselEntity
from vortexasdk.api.vessel_movement import VesselEvent, VesselMovement
from vortexasdk.api.eia_forecast import EIAForecast



CARGO_TIMESERIES_RESOURCE = "/time-series"

CARGO_MOVEMENTS_RESOURCE = "/cargo-movements/search"
VESSEL_MOVEMENTS_RESOURCE = "/vessel-movements/search"
EIA_FORECASTS_RESOURCE = "/search/eia-forecasts"

ATTRIBUTES_REFERENCE = "/reference/attributes"
GEOGRAPHIES_REFERENCE = "/reference/geographies"
VESSELS_REFERENCE = "/reference/vessels"
PRODUCTS_REFERENCE = "/reference/products"
CORPORATIONS_REFERENCE = "/reference/charterers"

API_FQDN = "https://api.vortexa.com"
API_URL = f"{API_FQDN}/v5"
